package index_data

import (
	"eta/eta_bridge/global"
	"eta/eta_bridge/models/jiayue"
	"eta/eta_bridge/models/response"
	"eta/eta_bridge/services/alarm_msg"
	"eta/eta_bridge/utils"
	"fmt"
	"strconv"
	"strings"
	"time"
)

// GetIndexFromJiaYue 获取嘉悦指标信息
func GetIndexFromJiaYue(indexCode string, sourceArr []string) (data *response.IndexResp, err error) {
	defer func() {
		if err != nil {
			global.LOG.Info("GetIndexFromJiaYue Err: " + err.Error())
			go alarm_msg.SendAlarmMsg("GetIndexFromJiaYue Err: "+err.Error(), 3)
			return
		}
	}()

	indexCond := ``
	indexPars := make([]interface{}, 0)
	// 指标存在无编码情况, 尝试转换indexCode为int查自增ID
	indexId, e := strconv.Atoi(indexCode)
	if e == nil && indexId > 0 {
		indexCond = ` (ID = :1 OR SOURCE_CODE = :1)`
		indexPars = append(indexPars, indexId, indexCode)
	} else {
		indexCond = ` SOURCE_CODE = :1`
		indexPars = append(indexPars, indexCode)
	}
	if len(sourceArr) > 0 {
		indexCond += ` AND SOURCE_TYPE IN (:1)`
		indexPars = append(indexPars, sourceArr)
	}

	indexes, e := jiayue.GetDictIndex(indexCond, indexPars, "")
	if e != nil {
		err = fmt.Errorf("GetDictIndex err: %s", e.Error())
		return
	}
	if len(indexes) <= 0 {
		global.LOG.Info(indexCode + "指标信息不存在")
		return
	}
	index := indexes[0]
	if index.Id <= 0 {
		err = fmt.Errorf("指标信息有误")
		return
	}

	var (
		indexData []jiayue.DictData
		dictMenu  []jiayue.DictCategory
	)
	data = FormatItem2Resp(index, indexData, dictMenu)
	return
}

// GetIndexAndDataFromJiaYue 获取指标和数据
func GetIndexAndDataFromJiaYue(indexCode, startDate, endDate string, sourceArr []string) (data *response.IndexResp, err error) {
	defer func() {
		if err != nil {
			global.LOG.Info("GetIndexAndDataFromJiaYue Err: " + err.Error())
			go alarm_msg.SendAlarmMsg("GetIndexAndDataFromJiaYue Err: "+err.Error(), 3)
			return
		}
	}()

	indexCond := ``
	indexPars := make([]interface{}, 0)
	// 指标存在无编码情况, 尝试转换indexCode为int查自增ID
	indexId, e := strconv.Atoi(indexCode)
	if e == nil && indexId > 0 {
		indexCond = ` (ID = :1 OR SOURCE_CODE = :1)`
		indexPars = append(indexPars, indexId, indexCode)
	} else {
		indexCond = ` SOURCE_CODE = :1`
		indexPars = append(indexPars, indexCode)
	}
	if len(sourceArr) > 0 {
		indexCond += ` AND SOURCE_TYPE IN (:1)`
		indexPars = append(indexPars, sourceArr)
	}

	indexes, e := jiayue.GetDictIndex(indexCond, indexPars, "")
	if e != nil {
		err = fmt.Errorf("GetDictIndex err: %s", e.Error())
		return
	}
	if len(indexes) <= 0 {
		global.LOG.Info(indexCode + "指标信息不存在")
		return
	}
	index := indexes[0]

	// 获取指标数据
	if index.Id <= 0 {
		err = fmt.Errorf("指标信息有误")
		return
	}
	if index.TableName == "" {
		err = fmt.Errorf("指标信息数据表名为空")
		return
	}

	dataCond := " INDEX_ID = :1"
	dataPars := make([]interface{}, 0)
	dataPars = append(dataPars, index.Id)
	if startDate != "" {
		dataCond += " AND INDEX_DATE >= TO_DATE (:1, 'yyyy-mm-dd')"
		dataPars = append(dataPars, startDate)
	}
	if endDate != "" {
		dataCond += " AND INDEX_DATE <= TO_DATE (:1, 'yyyy-mm-dd')"
		dataPars = append(dataPars, endDate)
	}
	indexData, e := jiayue.GetDictData(index.TableName, dataCond, dataPars)
	if e != nil {
		err = fmt.Errorf("GetDictData err: %s", e.Error())
		return
	}

	var dictMenu []jiayue.DictCategory
	data = FormatItem2Resp(index, indexData, dictMenu)
	return
}

// GetPageIndexesFromJiaYue 分页获取指标数据
func GetPageIndexesFromJiaYue(pageIndex, pageSize int, sourceArr []string, keyword, frequency string, sortField, sortRule int) (total int, result []jiayue.DictIndex, err error) {
	defer func() {
		if err != nil {
			global.LOG.Info("GetPageIndexesFromJiaYue Err: " + err.Error())
			go alarm_msg.SendAlarmMsg("GetPageIndexesFromJiaYue Err: "+err.Error(), 3)
			return
		}
	}()

	indexCond := ``
	indexPars := make([]interface{}, 0)
	// 筛选项
	if len(sourceArr) > 0 {
		indexCond += ` SOURCE_TYPE IN (:1)`
		indexPars = append(indexPars, sourceArr)
	}
	keyword = strings.TrimSpace(keyword)
	if keyword != "" {
		kw := fmt.Sprint("%", keyword, "%")
		indexCond += ` AND (ID LIKE :1 OR CODE LIKE :1 OR NAME LIKE :1)`
		indexPars = append(indexPars, kw, kw, kw)
	}
	if frequency != "" {
		indexCond += ` AND FREQUENCY = :1`
		indexPars = append(indexPars, frequency)
	}

	// 总记录数
	t, e := jiayue.GetDictIndexCount(indexCond, indexPars)
	if e != nil {
		err = fmt.Errorf("GetDictIndexCount err: %s", e.Error())
		return
	}
	total = t

	// 列表数据
	indexes, e := jiayue.GetDictPageIndex(indexCond, indexPars, pageIndex, pageSize, sortField, sortRule)
	if e != nil {
		err = fmt.Errorf("GetDictPageIndex err: %s", e.Error())
		return
	}
	result = indexes
	return
}

// GetNewIndexAndDataFromJiaYue 获取就近新增的指标和数据
func GetNewIndexAndDataFromJiaYue() (indexList []*response.IndexResp, err error) {
	defer func() {
		if err != nil {
			global.LOG.Info("GetIndexAndDataFromJiaYue Err: " + err.Error())
			go alarm_msg.SendAlarmMsg("GetIndexAndDataFromJiaYue Err: "+err.Error(), 3)
			return
		}
	}()

	indexCond := ``
	indexPars := make([]interface{}, 0)
	// TODO:查询两个小时之前的数据(待定)
	//timeBefore := time.Now().Local().Add(-2 * time.Hour)
	//indexCond += ` CREATE_TIME <= :1 `
	//indexPars = append(indexPars, timeBefore.Format(utils.FormatDateTime))

	// TODO:测试
	indexCond += ` UPDATE_TIME <= :1 `
	indexPars = append(indexPars, time.Date(2023, 11, 3, 17, 0, 0, 0, time.Local).Format(utils.FormatDateTime))
	sourceArr := []string{"webisite", "website", "website_gf"}
	if len(sourceArr) > 0 {
		indexCond += ` AND SOURCE_TYPE IN (:1)`
		indexPars = append(indexPars, sourceArr)
	}

	indexes, e := jiayue.GetDictIndex(indexCond, indexPars, "CREATE_TIME ASC")
	if e != nil {
		err = fmt.Errorf("GetDictIndex err: %s", e.Error())
		return
	}
	if len(indexes) <= 0 {
		global.LOG.Info("无新指标需要同步")
		return
	}

	// 每个指标对应的数据表不定, 所以这里还是用循环去查
	indexList = make([]*response.IndexResp, 0)
	for _, v := range indexes {
		// 指标数据
		dataCond := " INDEX_ID = :1"
		dataPars := make([]interface{}, 0)
		dataPars = append(dataPars, v.Id)
		indexData, e := jiayue.GetDictData(v.TableName, dataCond, dataPars)
		if e != nil {
			err = fmt.Errorf("GetDictData err: %s", e.Error())
			return
		}

		// 指标目录
		menuCond := ` R.INDEX_ID = :1`
		menuPars := make([]interface{}, 0)
		menuPars = append(menuPars, v.Id)
		menus, e := jiayue.GetDictCategory(menuCond, menuPars, "")
		if e != nil {
			err = fmt.Errorf("GetDictCategory err: %s", e.Error())
			return
		}

		item := FormatItem2Resp(v, indexData, menus)
		indexList = append(indexList, item)
	}
	return
}

// FormatItem2Resp 格式化指标响应体
func FormatItem2Resp(item jiayue.DictIndex, dictData []jiayue.DictData, dictMenu []jiayue.DictCategory) (res *response.IndexResp) {
	res = new(response.IndexResp)
	res.Id = item.Id
	res.IndexCode = item.Code
	res.IndexName = item.Name
	res.Unit = item.Unit
	res.Frequency = item.Frequency
	res.LastDate = item.DateLast
	res.LastUpdateTime = item.TimeLastUpdate
	res.Status = item.Status
	if len(dictData) > 0 {
		for _, d := range dictData {
			res.IndexData = append(res.IndexData, response.IndexData{
				Val:        d.IndexValue,
				DataTime:   d.IndexDate,
				UpdateTime: d.UpdateTime,
			})
		}
	}
	if len(dictMenu) > 0 {
		firstMenu := dictMenu[0]
		res.MenuData.Id = firstMenu.Id
		res.MenuData.Type = firstMenu.Type
		res.MenuData.Code = firstMenu.Code
		res.MenuData.Icon = firstMenu.Icon
		res.MenuData.Sort = firstMenu.Sorting
		res.MenuData.ParentId = firstMenu.ParentId
		res.MenuData.ParentName = firstMenu.ParentName
		res.MenuData.Path = firstMenu.Path
	}
	return
}
